@@ -137,6 +137,8 @@ class GitHubRepoSession(ProjectHolder):
         super(GitHubRepoSession, self).__init__()
         # dict holding repo/owner to feed contents of releases atom
         self.feed_contents = {}
+        # lazy loaded dict cache of /releases response keyed by tag, only first page
+        self.formal_releases_by_tag = None
         self.rate_limited_count = 0
         self.api_token = os.getenv("GITHUB_API_TOKEN")
         if not self.api_token:
@@ -181,39 +183,39 @@ class GitHubRepoSession(ProjectHolder):
                                           'Missing scopes?')
             raise ApiCredentialsError('Denied API access. Please set GITHUB_API_TOKEN env var '
                                       'as per https://github.com/dvershinin/lastversion#tips')
-        if r.status_code == 403:
-            if 'X-RateLimit-Reset' in r.headers and 'X-RateLimit-Remaining' in r.headers:
-                if self.rate_limited_count > 2:
-                    raise ApiCredentialsError(
-                        'API requests were denied after retrying {} times'.format(
-                            self.rate_limited_count)
-                    )
-                remaining = int(r.headers['X-RateLimit-Remaining'])
-                # 1 sec to account for skewed clock between GitHub and client
-                wait_for = float(r.headers['X-RateLimit-Reset']) - time.time() + 1.0
-                wait_for = math.ceil(wait_for)
-                if not remaining:
-                    # got 403, likely due to used quota
-                    if wait_for < 300:
-                        if wait_for < 0:
-                            log.warning(
-                                'Exceeded API quota. Repeating request because quota is about to '
-                                'be reinstated'
-                            )
-                        else:
-                            w = 'Waiting {} seconds for API quota reinstatement.'.format(wait_for)
-                            if "GITHUB_API_TOKEN" not in os.environ \
-                                    and 'GITHUB_TOKEN' not in os.environ:
-                                w = "{} {}".format(w, TOKEN_PRO_TIP)
-                            log.warning(w)
-                            time.sleep(wait_for)
-                        self.rate_limited_count = self.rate_limited_count + 1
-                        return self.get(url)
-                    raise ApiCredentialsError(
-                        'Exceeded API rate limit after waiting: {}'.format(
-                            r.json()['message'])
-                    )
-                return self.get(url)
+        if r.status_code == 403 and 'X-RateLimit-Reset' in r.headers \
+                and 'X-RateLimit-Remaining' in r.headers:
+            if self.rate_limited_count > 2:
+                raise ApiCredentialsError(
+                    'API requests were denied after retrying {} times'.format(
+                        self.rate_limited_count)
+                )
+            remaining = int(r.headers['X-RateLimit-Remaining'])
+            # 1 sec to account for skewed clock between GitHub and client
+            wait_for = float(r.headers['X-RateLimit-Reset']) - time.time() + 1.0
+            wait_for = math.ceil(wait_for)
+            if not remaining:
+                # got 403, likely due to used quota
+                if wait_for < 300:
+                    if wait_for < 0:
+                        log.warning(
+                            'Exceeded API quota. Repeating request because quota is about to '
+                            'be reinstated'
+                        )
+                    else:
+                        w = 'Waiting {} seconds for API quota reinstatement.'.format(wait_for)
+                        if "GITHUB_API_TOKEN" not in os.environ \
+                                and 'GITHUB_TOKEN' not in os.environ:
+                            w = "{} {}".format(w, TOKEN_PRO_TIP)
+                        log.warning(w)
+                        time.sleep(wait_for)
+                    self.rate_limited_count = self.rate_limited_count + 1
+                    return self.get(url)
+                raise ApiCredentialsError(
+                    'Exceeded API rate limit after waiting: {}'.format(
+                        r.json()['message'])
+                )
+            return self.get(url)
 
         if r.status_code == 403 and url != self.get_rate_limit_url():
             self.rate_limited_count = 0
@@ -224,6 +226,7 @@ class GitHubRepoSession(ProjectHolder):
         return self.get(url)
 
     def repo_query(self, uri):
+        """API query for a repository"""
         url = '{}/repos/{}{}'.format(self.api_base, self.repo, uri)
         return self.get(url)
 
@@ -231,7 +234,7 @@ class GitHubRepoSession(ProjectHolder):
         r = self.repo_query('/license?ref={}'.format(tag))
         if r.status_code == 200:
             # unfortunately, unlike /readme, API always returns *latest* license, ignoring tag
-            # we have to double check whether the license file exists "at release tag"
+            # we have to double-check whether the license file exists "at release tag"
             license_data = r.json()
             license_path = license_data['path']
             license_r = self.repo_query('/contents/{}?ref={}'.format(license_path, tag))
@@ -370,17 +373,42 @@ class GitHubRepoSession(ProjectHolder):
                 break
         return ret
 
+    def ensure_formal_releases_fetched(self):
+        """
+        Prime cache for dict of recent formal releases
+        this fetches /releases and allow quickly look up if a tag is marked as pre-release
+        """
+        if self.formal_releases_by_tag is None:
+            self.formal_releases_by_tag = {}
+            r = self.repo_query('/releases')
+            if r.status_code == 200:
+                for release in r.json():
+                    self.formal_releases_by_tag[release['tag_name']] = release
+
     def get_formal_release_for_tag(self, tag):
-        r = self.repo_query('/releases/tags/{}'.format(tag))
-        if r.status_code == 200:
-            # noinspection SpellCheckingInspection
-            return r.json()
-        return None
+        """Get formal release for a given tag, using cache from /releases"""
+        self.ensure_formal_releases_fetched()
+        # no releases in /releases means no
+        if self.formal_releases_by_tag and tag not in self.formal_releases_by_tag:
+            r = self.repo_query('/releases/tags/{}'.format(tag))
+            if r.status_code == 200:
+                self.formal_releases_by_tag[tag] = r.json()
+
+        return self.formal_releases_by_tag.get(tag)
 
-    # finding in tags requires paging through ALL of them, because the API does not list them
-    # in order of recency, thus this is very slow
-    # in: current release to be returned, output: newer release to be returned
     def find_in_tags(self, ret, pre_ok, major):
+        """
+        Find a more recent release in the /tags API endpoint.
+        Finding in /tags requires paging through ALL of them, because the API does not list them
+        in order of recency, thus this is very slow.
+        We need to check all tags commit dates simply because the most recent wins
+        We don't check tags which:
+
+        * marked pre-release in releases endpoints
+        * has a beta-like, non-version tag name
+
+        # in: current release to be returned, output: newer release to be returned
+        """
         r = self.repo_query('/tags?per_page=100')
         if r.status_code != 200:
             return None
@@ -469,7 +497,7 @@ class GitHubRepoSession(ProjectHolder):
 
     def get_latest(self, pre_ok=False, major=None):
         """
-        Gets latest release satisfying "prereleases are OK" or major/branch constraints
+        Get latest release satisfying "prereleases are OK" or major/branch constraints
         Strives to fetch formal API release if it exists, because it has useful information
         like assets.
         """
@@ -477,20 +505,12 @@ class GitHubRepoSession(ProjectHolder):
         # be returned
         ret = None
 
-        # always fetch /releases as this will allow us to quickly look up if a tag is marked as
-        # pre-release
-
         # then always get *all* tags through pagination
 
         # if pre not ok, filter out tags to check
 
         # if major, filter out tags to check for major
 
-        # we need to check all tags commit dates simply because the most recent wins
-        # we don't check tags which:
-        # * marked pre-release in releases endpoints
-        # * has a beta-like, non-version tag name
-
         feed_entries = self.get_releases_feed_entries()
         if feed_entries:
             for tag in feed_entries:
@@ -537,34 +557,22 @@ class GitHubRepoSession(ProjectHolder):
         if ret and ret['tag_date'].replace(tzinfo=None) > (datetime.utcnow() - timedelta(days=365)):
             return ret
 
+        log.info('Feed contained none or only tags older than 1 year. Switching to API')
+
         # only if we did not find desired stuff through feeds, we switch to using API :)
         # this may be required in cases
         # releases.atom has limited/no tags (#63), and all those are beta / invalid / non-versions
         # likewise, we want an older branch (major), which is not there in releases.atom
         # due to limited nature of data inside it
 
-        # releases/latest fetches only non-prerelease, non-draft, so it
-        # should not be used for hunting down pre-releases assets
-        if not pre_ok:
-            # https://stackoverflow.com/questions/28060116/which-is-more-reliable-for-github-api-conditional-requests-etag-or-last-modifie/57309763?noredirect=1#comment101114702_57309763
-            # ideally we disable ETag validation for this endpoint completely
-            r = self.repo_query('/releases/latest')
-            if r.status_code == 200:
-                formal_release = r.json()
-                version = self.sanitize_version(formal_release['tag_name'], pre_ok, major)
-                if version:
-                    ret = self.set_matching_formal_release(ret, formal_release, version,
-                                                           pre_ok, 'releases-latest')
-        else:
-            r = self.repo_query('/releases')
-            if r.status_code == 200:
-                for release in r.json():
-                    tag_name = release['tag_name']
-                    version = self.sanitize_version(tag_name, pre_ok, major)
-                    if not version:
-                        continue
-                    if not ret or version > ret['version']:
-                        ret = self.set_matching_formal_release(ret, release, version, pre_ok)
+        self.ensure_formal_releases_fetched()
+        for tag_name in self.formal_releases_by_tag:
+            release = self.formal_releases_by_tag[tag_name]
+            version = self.sanitize_version(tag_name, pre_ok, major)
+            if not version:
+                continue
+            if not ret or version > ret['version']:
+                ret = self.set_matching_formal_release(ret, release, version, pre_ok)
 
         if self.having_asset:
             # only formal releases which we enumerated above already, have assets
