@@ -1,5 +1,6 @@
 import logging
 
+from .GiteaRepoSession import GiteaRepoSession
 from .BitBucketRepoSession import BitBucketRepoSession
 from .FeedRepoSession import FeedRepoSession
 from .GitHubRepoSession import GitHubRepoSession
@@ -31,8 +32,11 @@ class HolderFactory:
         'wiki': WikipediaRepoSession,
         'system': SystemRepoSession,
         'wp': WordPressPluginRepoSession,
+        'gitea': GiteaRepoSession
     }
 
+    DEFAULT_HOLDER = 'github'
+
     @staticmethod
     def guess_from_homepage(repo, hostname):
         # repo auto-discovery failed for detected/default provider
@@ -50,11 +54,11 @@ class HolderFactory:
     @staticmethod
     # go through subclasses in order to find the one that is holding a given project
     # repo is either complete URL or a name allowing to identify a single project
-    def get_instance_for_repo(repo, only=None, at=None):
+    def get_instance_for_repo(repo, at=None):
         """Find the right hosting for this repo."""
         if at == 'helm_chart' or (at and '/' not in repo):
             return HolderFactory.HOLDERS[at](repo, hostname=None)
-        holder_class = HolderFactory.HOLDERS['github']
+        holder_class = HolderFactory.HOLDERS[HolderFactory.DEFAULT_HOLDER]
         hostname = None
         known_repo = None
 
@@ -84,12 +88,14 @@ class HolderFactory:
                 raise BadProjectError(
                     'No project found. Could not guess a repo from homepage'
                 )
+            
         if known_repo and 'branches' in known_repo:
             holder.set_branches(known_repo['branches'])
+            
         if known_repo and 'only' in known_repo:
             holder.set_only(known_repo['only'])
-        if only:
-            holder.set_only(only)
+
         if known_repo and 'release_url_format' in known_repo:
             holder.RELEASE_URL_FORMAT = known_repo['release_url_format']
+
         return holder
